@@ -24,7 +24,8 @@ import numpy as np
 import matplotlib.pyplot as plt
 import matplotlib.ticker as ticker
 from .sdynpy_array import SdynpyArray
-from .sdynpy_coordinate import CoordinateArray,outer_product,coordinate_array
+from .sdynpy_coordinate import CoordinateArray, outer_product
+
 
 class Matrix(SdynpyArray):
     """Matrix with degrees of freedom stored for better bookkeeping.
@@ -32,10 +33,8 @@ class Matrix(SdynpyArray):
     Use the matrix helper function to create the object.
     """
 
-    data_dtype = [('node', 'uint64'), ('direction', 'int8')]
-    
     @staticmethod
-    def data_dtype(rows,columns):
+    def data_dtype(rows, columns, is_complex=False):
         """
         Data type of the underlying numpy structured array for real shapes
 
@@ -43,7 +42,7 @@ class Matrix(SdynpyArray):
         ----------
         rows : int
             Number of rows in the matrix
-        columns : int 
+        columns : int
             Number of columns in the matrix
 
         Returns
@@ -54,56 +53,56 @@ class Matrix(SdynpyArray):
 
         """
         return [
-            ('matrix', 'float64', (rows,columns)),
+            ('matrix', 'complex128' if is_complex else 'float64', (rows, columns)),
             ('row_coordinate', CoordinateArray.data_dtype, (rows,)),
             ('column_coordinate', CoordinateArray.data_dtype, (columns,)),
         ]
-    
+
     """Datatype for the underlying numpy structured array"""
 
-    def __new__(subtype, shape, nrows, ncols, buffer=None, offset=0,
+    def __new__(subtype, shape, nrows, ncols, is_complex=False, buffer=None, offset=0,
                 strides=None, order=None):
         # Create the ndarray instance of our type, given the usual
         # ndarray input arguments.  This will call the standard
         # ndarray constructor, but return an object of our type.
         # It also triggers a call to InfoArray.__array_finalize__
         obj = super(Matrix, subtype).__new__(subtype, shape,
-                                             Matrix.data_dtype(nrows,ncols),
+                                             Matrix.data_dtype(nrows, ncols, is_complex),
                                              buffer, offset, strides,
                                              order)
         # Finally, we must return the newly created object:
         return obj
-    
+
     @property
     def coordinate(self):
         """
         Returns the full coordinate array for the matrix
         """
-        return outer_product(self.row_coordinate,self.column_coordinate)
-    
+        return outer_product(self.row_coordinate, self.column_coordinate)
+
     @coordinate.setter
-    def coordinate(self,value):
+    def coordinate(self, value):
         raise RuntimeError('Cannot set coordinate directly.  Set row_coordinate or column_coordinate instead.')
-    
+
     @property
     def num_coordinate_rows(self):
         """
         Returns the number of coordinate rows
         """
         return self.matrix.shape[-2]
-    
+
     @property
     def num_coordinate_columns(self):
         """
         Returns the number of coordinate columns
         """
         return self.matrix.shape[-1]
-    
+
     def __setitem__(self, key, value):
         if type(key) is tuple and len(key) == 2 and any([(type(val) is CoordinateArray) for val in key]):
             # Get indices corresponding to the coordinates
-            row_request,column_request = key
-            if row_request is None or (type(row_request) == slice and row_request == slice(None)) or type(row_request) == type(Ellipsis):
+            row_request, column_request = key
+            if row_request is None or (isinstance(row_request, slice) and row_request == slice(None)) or isinstance(row_request, type(Ellipsis)):
                 matrix_row_indices = np.arange(self.num_coordinate_rows)
                 request_row_indices = np.arange(self.num_coordinate_rows)
                 row_multiplications = np.ones(self.num_coordinate_rows)
@@ -134,7 +133,7 @@ class Matrix(SdynpyArray):
                 # # Invert the indices to return the dofs in the correct order as specified in keys
                 # inverse_row_indices = np.zeros(matrix_row_indices.shape, dtype=int)
                 # inverse_row_indices[matrix_row_indices] = np.arange(len(matrix_row_indices))
-            if column_request is None or (type(column_request) == slice and column_request == slice(None)) or type(column_request) == type(Ellipsis):
+            if column_request is None or (isinstance(column_request, slice) and column_request == slice(None)) or isinstance(column_request, type(Ellipsis)):
                 matrix_col_indices = np.arange(self.num_coordinate_columns)
                 request_col_indices = np.arange(self.num_coordinate_columns)
                 col_multiplications = np.ones(self.num_coordinate_columns)
@@ -166,18 +165,18 @@ class Matrix(SdynpyArray):
             value = np.array(value)
             value = np.broadcast_to(value,
                                     value.shape[:-2] + (len(consistent_row_coordinates),
-                                     len(consistent_col_coordinates)))
-            self.matrix[..., matrix_row_indices[:,np.newaxis],matrix_col_indices] = (
-                value[...,request_row_indices[:,np.newaxis],request_col_indices]
-                * row_multiplications[:,np.newaxis] * col_multiplications)
+                                                        len(consistent_col_coordinates)))
+            self.matrix[..., matrix_row_indices[:, np.newaxis], matrix_col_indices] = (
+                value[..., request_row_indices[:, np.newaxis], request_col_indices]
+                * row_multiplications[:, np.newaxis] * col_multiplications)
         else:
-            super().__setitem__(key,value)
-    
+            super().__setitem__(key, value)
+
     def __getitem__(self, key):
         if type(key) is tuple and len(key) == 2 and any([(type(val) is CoordinateArray) for val in key]):
             # Get indices corresponding to the coordinates
-            row_request,column_request = key
-            if row_request is None or (type(row_request) == slice and row_request == slice(None)) or type(row_request) == type(Ellipsis):
+            row_request, column_request = key
+            if row_request is None or (isinstance(row_request, slice) and row_request == slice(None)) or isinstance(row_request, type(Ellipsis)):
                 matrix_row_indices = np.arange(self.num_coordinate_rows)
                 inverse_row_indices = np.arange(self.num_coordinate_rows)
                 row_multiplications = np.ones(self.num_coordinate_rows)
@@ -208,7 +207,7 @@ class Matrix(SdynpyArray):
                 # Invert the indices to return the dofs in the correct order as specified in keys
                 inverse_row_indices = np.zeros(request_row_indices.shape, dtype=int)
                 inverse_row_indices[request_row_indices] = np.arange(len(request_row_indices))
-            if column_request is None or (type(column_request) == slice and column_request == slice(None)) or type(column_request) == type(Ellipsis):
+            if column_request is None or (isinstance(column_request, slice) and column_request == slice(None)) or isinstance(column_request, type(Ellipsis)):
                 matrix_col_indices = np.arange(self.num_coordinate_columns)
                 inverse_col_indices = np.arange(self.num_coordinate_columns)
                 col_multiplications = np.ones(self.num_coordinate_columns)
@@ -237,21 +236,21 @@ class Matrix(SdynpyArray):
                 # Invert the indices to return the dofs in the correct order as specified in keys
                 inverse_col_indices = np.zeros(request_col_indices.shape, dtype=int)
                 inverse_col_indices[request_col_indices] = np.arange(len(request_col_indices))
-            return_value = self.matrix[..., matrix_row_indices[:,np.newaxis], matrix_col_indices] * row_multiplications[:,np.newaxis] * col_multiplications
-            return_value = return_value[..., inverse_row_indices[:,np.newaxis],inverse_col_indices]
+            return_value = self.matrix[..., matrix_row_indices[:, np.newaxis], matrix_col_indices] * row_multiplications[:, np.newaxis] * col_multiplications
+            return_value = return_value[..., inverse_row_indices[:, np.newaxis], inverse_col_indices]
             return return_value
         else:
             output = super().__getitem__(key)
-            if key in ['row_coordinate','column_coordinate']:
+            if isinstance(key, str) and key in ['row_coordinate', 'column_coordinate']:
                 return output.view(CoordinateArray)
             else:
                 return output
-            
+
     def __repr__(self):
         return '\n\n'.join(['matrix = \n'+repr(self.matrix),
-                          'row coordinates = \n'+repr(self.row_coordinate),
-                          'column coordinates = \n'+repr(self.column_coordinate)])
-    
+                            'row coordinates = \n'+repr(self.row_coordinate),
+                            'column coordinates = \n'+repr(self.column_coordinate)])
+
     def argsort_coordinate(self):
         """
         Returns indices used to sort the coordinates on the rows and columns
@@ -264,13 +263,13 @@ class Matrix(SdynpyArray):
             Indices used to sort the column coordinates
 
         """
-        row_indices = np.empty(self.row_coordinate.shape,dtype=int)
-        column_indices = np.empty(self.column_coordinate.shape,dtype=int)
-        for key,matrix in self.ndenumerate():
+        row_indices = np.empty(self.row_coordinate.shape, dtype=int)
+        column_indices = np.empty(self.column_coordinate.shape, dtype=int)
+        for key, matrix in self.ndenumerate():
             row_indices[key] = np.argsort(self.row_coordinate[key])
             column_indices[key] = np.argsort(self.column_coordinate[key])
-        return row_indices[...,:,np.newaxis],column_indices[...,np.newaxis,:]
-    
+        return row_indices[..., :, np.newaxis], column_indices[..., np.newaxis, :]
+
     def sort_coordinate(self):
         """
         Returns a copy of the Matrix with coordinate sorted
@@ -283,13 +282,13 @@ class Matrix(SdynpyArray):
         """
         sorting = self.argsort_coordinate()
         return_val = self.copy()
-        for key,matrix in self.ndenumerate():
-            return_val.matrix[key] = self.matrix[key][...,sorting[0][key],sorting[1][key]]
-            return_val.row_coordinate = self.row_coordinate[...,sorting[0][key][:,0]]
-            return_val.column_coordinate = self.column_coordinate[...,sorting[1][key][0,:]]
+        for key, matrix in self.ndenumerate():
+            return_val.matrix[key] = self.matrix[key][..., sorting[0][key], sorting[1][key]]
+            return_val.row_coordinate = self.row_coordinate[..., sorting[0][key][:, 0]]
+            return_val.column_coordinate = self.column_coordinate[..., sorting[1][key][0, :]]
         return return_val
-            
-    def plot(self,ax = None,show_colorbar=True,**imshow_args):
+
+    def plot(self, ax=None, show_colorbar=True, **imshow_args):
         """
         Plots the matrix with coordinates labeled
 
@@ -312,7 +311,8 @@ class Matrix(SdynpyArray):
         if self.size > 1:
             raise ValueError('Cannot plot more than one Matrix object simultaneously')
         if ax is None:
-            fig,ax = plt.subplots()
+            fig, ax = plt.subplots()
+
         @ticker.FuncFormatter
         def row_formatter(x, pos):
             if int(x) < 0 or int(x) >= len(self.row_coordinate):
@@ -321,6 +321,7 @@ class Matrix(SdynpyArray):
                 return str(self.row_coordinate[int(x)])
             except IndexError:
                 return ''
+
         @ticker.FuncFormatter
         def col_formatter(x, pos):
             if int(x) < 0 or int(x) >= len(self.column_coordinate):
@@ -329,16 +330,16 @@ class Matrix(SdynpyArray):
                 return str(self.column_coordinate[int(x)])
             except IndexError:
                 return ''
-        out = ax.imshow(self.matrix,**imshow_args)
+        out = ax.imshow(self.matrix, **imshow_args)
         if show_colorbar:
             plt.colorbar(out, ax=ax)
         ax.xaxis.set_major_formatter(col_formatter)
         ax.xaxis.set_major_locator(ticker.MaxNLocator(integer=True))
         ax.yaxis.set_major_formatter(row_formatter)
         ax.yaxis.set_major_locator(ticker.MaxNLocator(integer=True))
-        
+
     @classmethod
-    def eye(cls,row_coordinate,column_coordinate = None):
+    def eye(cls, row_coordinate, column_coordinate=None):
         """
         Creates an identity matrix with the specified coordinates
 
@@ -358,11 +359,31 @@ class Matrix(SdynpyArray):
         """
         if column_coordinate is None:
             column_coordinate = row_coordinate
-        return matrix(np.eye(row_coordinate.shape[-1],column_coordinate.shape[-1]),
-                      row_coordinate,column_coordinate)
+        return matrix(np.eye(row_coordinate.shape[-1], column_coordinate.shape[-1]),
+                      row_coordinate, column_coordinate)
     
-def matrix(matrix,row_coordinate,column_coordinate, buffer=None, offset=0,
-            strides=None, order=None):
+    def pinv(self, **pinv_params):
+        """
+        Creates the pseudoinverse of the matrix
+
+        Parameters
+        ----------
+        **pinv_params : various
+            Extra keyword arguments are passed directly to np.linalg.pinv
+
+        Returns
+        -------
+        Matrix
+            A matrix consisting of the pseudoinverse of the original matrix
+
+        """
+        mat = np.linalg.pinv(self.matrix,**pinv_params)
+        return matrix(matrix=mat,row_coordinate = self.column_coordinate,
+                      column_coordinate = self.row_coordinate)
+
+
+def matrix(matrix, row_coordinate, column_coordinate, buffer=None, offset=0,
+           strides=None, order=None):
     """
     Create a matrix object
 
@@ -382,11 +403,11 @@ def matrix(matrix,row_coordinate,column_coordinate, buffer=None, offset=0,
 
     """
     shape = matrix.shape[:-2]
-    nrow,ncol = matrix.shape[-2:]
-    m = Matrix(shape,nrow,ncol,
+    nrow, ncol = matrix.shape[-2:]
+    m = Matrix(shape, nrow, ncol, np.iscomplexobj(matrix),
                buffer=buffer, offset=offset,
                strides=strides, order=order)
     m.row_coordinate = row_coordinate
     m.column_coordinate = column_coordinate
     m.matrix = matrix
-    return m
\ No newline at end of file
+    return m
