@@ -26,17 +26,17 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 """
 
 import numpy as np
-from scipy.linalg import eigh
 from .sdynpy_coordinate import CoordinateArray, from_nodelist, outer_product, coordinate_array
 from ..fem.sdynpy_beam import beamkm, rect_beam_props
-from ..fem.sdynpy_exodus import Exodus, ExodusInMemory
+from ..fem.sdynpy_exodus import Exodus, ExodusInMemory, reduce_exodus_to_surfaces, read_sierra_matlab_matrix_file, read_sierra_matlab_map_file
 from ..signal_processing import frf as spfrf
 from ..signal_processing import generator
-from scipy.linalg import eigh, block_diag, null_space
+from scipy.linalg import eigh, block_diag, null_space, eig
 from scipy.signal import lsim, StateSpace, resample, butter, filtfilt
 import copy
 import netCDF4 as nc4
 import matplotlib.pyplot as plt
+import warnings
 
 
 class System:
@@ -108,7 +108,7 @@ class System:
         # Check symmetry
         if not np.allclose(mass, mass.T):
             raise ValueError('mass matrix must be symmetric')
-        if not np.allclose(stiffness, stiffness.T):
+        if not np.allclose(stiffness, stiffness.T, atol=1e-6*stiffness.max()):
             raise ValueError('stiffness matrix must be symmetric')
         if not np.allclose(damping, damping.T):
             raise ValueError('damping matrix must be symmetric')
@@ -260,7 +260,8 @@ class System:
         """Get the number of physical degrees of freedom of the system"""
         return self.transformation.shape[0]
 
-    def to_state_space(self, output_displacement=True, output_velocity=True, output_acceleration=True, output_force=True):
+    def to_state_space(self, output_displacement=True, output_velocity=True, output_acceleration=True, output_force=True,
+                       response_coordinates=None, input_coordinates=None):
         """
         Compute the state space representation of the system
 
@@ -292,45 +293,59 @@ class System:
 
         """
         ndofs = self.ndof
-        tdofs = self.ndof_transformed
         M = self.M
         K = self.K
         C = self.C
-        phi = self.transformation
+        if response_coordinates is None:
+            phi_response = self.transformation
+        elif isinstance(response_coordinates, str) and response_coordinates == 'state':
+            # If you want the state variables back, the transformation is the
+            # identity matrix.
+            phi_response = np.eye(self.transformation.shape[-1])
+        else:
+            phi_response = self.transformation_matrix_at_coordinates(response_coordinates)
+        if input_coordinates is None:
+            phi_input = self.transformation
+        else:
+            phi_input = self.transformation_matrix_at_coordinates(input_coordinates)
+        tdofs_response = phi_response.shape[0]
+        tdofs_input = phi_input.shape[0]
         # A = [[     0,     I],
         #      [M^-1*K,M^-1*C]]
 
         A_state = np.block([[np.zeros((ndofs, ndofs)), np.eye(ndofs)],
                             [-np.linalg.solve(M, K), -np.linalg.solve(M, C)]])
 
-        # B = [[     0,  M^-1]]
+        # B = [[     0],
+        #      [  M^-1]]
 
-        B_state = np.block([[np.zeros((ndofs, tdofs))], [np.linalg.solve(M, phi.T)]])
+        B_state = np.block([[np.zeros((ndofs, tdofs_input))],
+                            [np.linalg.solve(M, phi_input.T)]])
 
         # C = [[     I,     0],   # Displacements
         #      [     0,     I],   # Velocities
         #      [M^-1*K,M^-1*C],   # Accelerations
         #      [     0,     0]]   # Forces
 
-        C_state = np.block([[phi, np.zeros((tdofs, ndofs))],
-                            [np.zeros((tdofs, ndofs)), phi],
-                            [-phi @ np.linalg.solve(M, K), -phi @ np.linalg.solve(M, C)],
-                            [np.zeros((tdofs, ndofs)), np.zeros((tdofs, ndofs))]])
+        C_state = np.block([[phi_response, np.zeros((tdofs_response, ndofs))],
+                            [np.zeros((tdofs_response, ndofs)), phi_response],
+                            [-phi_response @ np.linalg.solve(M, K), -phi_response @ np.linalg.solve(M, C)],
+                            [np.zeros((tdofs_input, ndofs)), np.zeros((tdofs_input, ndofs))]])
 
         # D = [[     0],   # Displacements
         #      [     0],   # Velocities
         #      [  M^-1],   # Accelerations
         #      [     I]]   # Forces
 
-        D_state = np.block([[np.zeros((tdofs, tdofs))],
-                            [np.zeros((tdofs, tdofs))],
-                            [phi @ np.linalg.solve(M, phi.T)],
-                            [np.eye(tdofs)]])
-        displacement_indices = np.arange(tdofs)
-        velocity_indices = np.arange(tdofs) + tdofs
-        acceleration_indices = np.arange(tdofs) + 2 * tdofs
-        force_indices = np.arange(tdofs) + 3 * tdofs
-        output_indices = np.zeros(4 * tdofs, dtype=bool)
+        D_state = np.block([[np.zeros((tdofs_response, tdofs_input))],
+                            [np.zeros((tdofs_response, tdofs_input))],
+                            [phi_response @ np.linalg.solve(M, phi_input.T)],
+                            [np.eye(tdofs_input)]])
+        displacement_indices = np.arange(tdofs_response)
+        velocity_indices = np.arange(tdofs_response) + tdofs_response
+        acceleration_indices = np.arange(tdofs_response) + 2 * tdofs_response
+        force_indices = np.arange(tdofs_input) + 3 * tdofs_response
+        output_indices = np.zeros(3 * tdofs_response + tdofs_input, dtype=bool)
         if output_displacement:
             output_indices[displacement_indices] = True
         if output_velocity:
@@ -343,7 +358,7 @@ class System:
         D_state = D_state[output_indices]
         return A_state, B_state, C_state, D_state
 
-    def time_integrate(self, forces, dt, responses=None, references=None,
+    def time_integrate(self, forces, dt=None, responses=None, references=None,
                        displacement_derivative=2, initial_state=None,
                        integration_oversample=1):
         """
@@ -351,18 +366,23 @@ class System:
 
         Parameters
         ----------
-        forces : np.ndarray
+        forces : np.ndarray or TimeHistoryArray
             The forces applied to the system, which should be a signal with
-            time-step `dt`
-        dt : float
-            The timestep used for the integration.
+            time-step `dt`.  If a `TimeHistoryArray` is passed, then `dt` and
+            `references` will be taken from the `TimeHistoryArray`, and the
+            arguments will be ignored.
+        dt : float, optional
+            The timestep used for the integration.  Must be specified if `forces`
+            is a ndarray.  If `forces` is a TimeHistoryArray, then this argument
+            is ignored.
         responses : CoordinateArray, optional
             Coordinates at which responses are desired. The default is all
             responses.
         references : CoordinateArray, optional
             Coordinates at which responses are input. The default is all
             references.  Must be the same size as the number of rows in the
-            `forces` array.
+            `forces` array.  If `forces` is a `TimeHistoryArray`, then this
+            argument is ignored.
         displacement_derivative : int, optional
             The derivative of the displacement that the output will be
             represented as.  A derivative of 0 means displacements will be
@@ -384,36 +404,24 @@ class System:
             The forces applied to the system as a TimeHistoryArray
 
         """
-        from .sdynpy_data import data_array, FunctionTypes
+        from .sdynpy_data import data_array, FunctionTypes, TimeHistoryArray
+        if isinstance(forces, TimeHistoryArray):
+            dt = forces.abscissa_spacing
+            references = forces.coordinate[..., 0]
+            forces = forces.ordinate
+        else:
+            if dt is None:
+                raise ValueError('`dt` must be specified if `forces` is not a `TimeHistoryArray`')
+        if responses is not None and not isinstance(responses, str):
+            responses = np.atleast_1d(responses)
+        if references is not None:
+            references = np.atleast_1d(references)
         A, B, C, D = self.to_state_space(displacement_derivative == 0,
                                          displacement_derivative == 1,
                                          displacement_derivative == 2,
-                                         False)
-        # Partition references into shape matrix
-        if references is None:
-            reference_indices = np.arange(self.coordinate.size)
-            reference_signs = np.ones(self.coordinate.size, 'int8')
-        else:
-            reference_indices = np.array([np.where(abs(self.coordinate) == abs(coord))[
-                                         0][0] for coord in np.atleast_1d(references)])
-            reference_signs = np.sign(
-                self.coordinate[reference_indices].direction) * np.sign(np.atleast_1d(references).direction)
-            reference_signs[reference_signs == 0] = 1
-        if responses is None:
-            response_indices = np.arange(self.coordinate.size)
-            response_signs = np.ones(self.coordinate.size, 'int8')
-        else:
-            response_indices = np.array([np.where(abs(self.coordinate) == abs(coord))[
-                                        0][0] for coord in np.atleast_1d(responses)])
-            response_signs = np.sign(
-                self.coordinate[response_indices].direction) * np.sign(np.atleast_1d(responses).direction)
-            response_signs[response_signs == 0] = 1
-        C = C[response_indices, :]
-        C = response_signs[:, np.newaxis] * C
-        D = D[response_indices[:, np.newaxis], reference_indices[np.newaxis, :]]
-        D = response_signs[:, np.newaxis] * reference_signs[np.newaxis, :] * D
-        B = B[:, reference_indices]
-        B = reference_signs[np.newaxis, :] * B
+                                         False,
+                                         responses, references
+                                         )
         forces = np.atleast_2d(forces)
         times = np.arange(forces.shape[-1]) * dt
         linear_system = StateSpace(A, B, C, D)
@@ -422,13 +430,16 @@ class System:
         times_out, time_response, x_out = lsim(linear_system, forces.T, times, initial_state)
         if time_response.ndim == 1:
             time_response = time_response[:, np.newaxis]
-        response_coordinate = self.coordinate[response_indices].copy()
-        response_coordinate.direction *= response_signs
+        if responses is None:
+            response_coordinate = self.coordinate.copy()
+        elif isinstance(responses, str) and responses == 'state':
+            response_coordinate = coordinate_array(np.arange(self.ndof), 0)
+        else:
+            response_coordinate = responses
         response_array = data_array(FunctionTypes.TIME_RESPONSE,
                                     times, time_response.T,
                                     response_coordinate[:, np.newaxis])
-        reference_coordinate = self.coordinate[reference_indices].copy()
-        reference_coordinate.direction *= reference_signs
+        reference_coordinate = self.coordinate.copy() if references is None else references
         reference_array = data_array(FunctionTypes.TIME_RESPONSE,
                                      times, np.atleast_2d(forces),
                                      reference_coordinate[:, np.newaxis])
@@ -470,9 +481,9 @@ class System:
 
         """
         if complex_modes is False:
-            if not num_modes is None:
+            if num_modes is not None:
                 num_modes = [0, int(num_modes) - 1]
-            if not maximum_frequency is None:
+            if maximum_frequency is not None:
                 maximum_frequency = (2 * np.pi * maximum_frequency)**2
                 maximum_frequency = [-maximum_frequency, maximum_frequency]  # Convert to eigenvalue
             lam, phi = eigh(self.K, self.M, subset_by_index=num_modes,
@@ -495,7 +506,69 @@ class System:
             else:
                 return System(self.coordinate, np.eye(freq.size), np.diag((2 * np.pi * freq)**2), np.diag(2 * (2 * np.pi * freq) * damping), phi)
         else:
-            raise NotImplementedError('Complex Modes are not implemented yet!')
+            if self.ndof > 1000:
+                warnings.warn('The complex mode implementation currently computes all eigenvalues and eigenvectors, which may take a long time for large systems.')
+            # For convenience, assign a zeros matrix
+            Z = np.zeros(self.M.shape)
+            A = np.block([[     Z, self.M],
+                          [self.M, self.C]])
+            B = np.block([[-self.M,      Z],
+                          [      Z, self.K]])
+            lam, E = eig(-B,A)
+            # Sort the eigenvalues such that they are increasing in frequency
+            isort = np.argsort(np.abs(lam))
+            lam = lam[isort]
+            E = E[:,isort]
+            # Eigenvalues will be in complex conjugate pairs.  Let's only keep
+            # the ones that are greater than zero.
+            keep = lam.imag > 0
+            lam = lam[keep]
+            E = E[:,keep]
+            if A.shape[0]//2 != lam.size:
+                warnings.warn('The complex mode implementation currently does not do well with rigid body modes (0 Hz frequencies).  Compute them from a real-modes solution then transform to complex.')
+            # Cull values we don't want
+            if num_modes is not None:
+                lam = lam[:num_modes]
+                E = E[:,:num_modes]
+            if maximum_frequency is not None:
+                keep = np.abs(lam) <= maximum_frequency*2*np.pi
+                lam = lam[keep]
+                E = E[:,keep]
+            # Mass normalize the mode shapes
+            E = E/np.sqrt(np.einsum('ji,jk,ki->i',E,A,E))
+            # Find repeated eigenvalues where the eigenvectors are not orthogonal
+            # TODO: Might have to do some orthogonalization for repeated
+            # eigenvalues
+            # A_modal = np.einsum('ji,jk,kl->il',E,A,E)
+            # Extract just the displacement partition
+            psi = E[E.shape[0]//2:,:]
+            # Add in a transformation to get to physical dofs
+            psi_t = self.transformation @ psi
+            frequency = np.abs(lam)/(2*np.pi)
+            damping = -np.real(lam)/np.abs(lam)
+            if return_shape:
+                from .sdynpy_shape import shape_array
+                return shape_array(self.coordinate, psi.T, frequency, damping)
+            else:
+                warnings.warn('Complex Modes will in general not diagonalize the system M, C, and K, matrices.')
+                return System(self.coordinate,
+                              psi.T@self.M@psi,
+                              psi.T@self.K@psi.T,
+                              psi.T@self.C@psi.T,
+                              psi_t)
+            
+
+    def transformation_shapes(self, shape_indices=None):
+        from .sdynpy_shape import shape_array
+        if shape_indices is None:
+            shape_indices = slice(None)
+        shape_matrix = self.transformation[:, shape_indices]
+        return shape_array(self.coordinate, shape_matrix.T,
+                           frequency=0, damping=0)
+
+    def remove_transformation(self):
+        return System(coordinate_array(np.arange(self.ndof)+1,0),
+                      self.mass.copy(),self.stiffness.copy(),self.damping.copy())
 
     def frequency_response(self, frequencies, responses=None, references=None,
                            displacement_derivative=0):
@@ -550,6 +623,33 @@ class System:
                          outer_product(output_coordinates, input_coordinates))
         return frf
 
+    def assign_modal_damping(self, damping_ratios):
+        """
+        Assigns a damping matrix to the system that results in equivalent
+        modal damping
+
+        Parameters
+        ----------
+        damping_ratios : ndarray
+            An array of damping values to assign to the system
+
+        Returns
+        -------
+        None.
+
+        """
+        damping_ratios = np.array(damping_ratios)
+        if damping_ratios.ndim == 1:
+            shapes = self.eigensolution(num_modes=damping_ratios.size)
+        else:
+            shapes = self.eigensolution()
+        shapes.damping = damping_ratios
+        # Compute the damping matrix
+        modal_system = shapes.system()
+        shape_pinv = np.linalg.pinv(modal_system.transformation.T)
+        full_damping_matrix = shape_pinv@modal_system.damping@shape_pinv.T
+        self.damping[:] = full_damping_matrix
+
     def save(self, filename):
         """
         Saves the system to a file
@@ -591,7 +691,7 @@ class System:
     def __neg__(self):
         new_system = copy.deepcopy(self)
         new_system.mass *= -1
-        new_system.stiffness * - -1
+        new_system.stiffness *= -1
         new_system.damping *= -1
         return new_system
 
@@ -792,7 +892,7 @@ class System:
         constraint_matrix = []
         for constraint_dof_0, constraint_dof_1 in dof_pairs:
             constraint = self.transformation_matrix_at_coordinates(constraint_dof_0)
-            if not constraint_dof_1 is None:
+            if constraint_dof_1 is not None:
                 constraint -= self.transformation_matrix_at_coordinates(constraint_dof_1)
             constraint_matrix.append(constraint)
         constraint_matrix = np.concatenate(constraint_matrix, axis=0)
@@ -814,7 +914,7 @@ class System:
         connection_dofs_0 : CoordinateArray
             Array of coordinates to use in the constraints
         connection_dofs_1 : CoordinateArray, optional
-            Array of coordinates to constrain to the coordinates in 
+            Array of coordinates to constrain to the coordinates in
             `connection_dofs_0`. If not specified, the `connection_dofs_0`
             degrees of freedom will be constrained to ground.
         rcond : float, optional
@@ -835,7 +935,7 @@ class System:
         shape_matrix_0 = constraint_shapes[connection_dofs_0].T
         transform_matrix_0 = self.transformation_matrix_at_coordinates(connection_dofs_0)
         constraint_matrix = np.linalg.lstsq(shape_matrix_0, transform_matrix_0)[0]
-        if not connection_dofs_1 is None:
+        if connection_dofs_1 is not None:
             shape_matrix_1 = constraint_shapes[connection_dofs_1].T
             transform_matrix_1 = self.transformation_matrix_at_coordinates(connection_dofs_1)
             constraint_matrix -= np.linalg.lstsq(shape_matrix_1, transform_matrix_1)[0]
@@ -861,7 +961,7 @@ class System:
         """
         Sets the damping matrix to a proportion of the mass and stiffness matrices.
 
-        The damping matrix will be set to `mass_fraction*self.mass + 
+        The damping matrix will be set to `mass_fraction*self.mass +
         stiffness_fraction*self.stiffness`
 
         Parameters
@@ -1048,7 +1148,7 @@ class System:
             keep_dofs = self.get_indices_by_coordinate(coordinates)
         else:
             keep_dofs = np.array(coordinates)
-        discard_dofs = np.array([i for i in range(self.ndof) if not i in keep_dofs])
+        discard_dofs = np.array([i for i in range(self.ndof) if i not in keep_dofs])
         I_a = np.eye(keep_dofs.size)
         K_dd = self.stiffness[discard_dofs[:, np.newaxis],
                               discard_dofs]
@@ -1087,7 +1187,7 @@ class System:
             keep_dofs = self.get_indices_by_coordinate(coordinates)
         else:
             keep_dofs = np.array(coordinates)
-        discard_dofs = np.array([i for i in range(self.ndof) if not i in keep_dofs])
+        discard_dofs = np.array([i for i in range(self.ndof) if i not in keep_dofs])
         I_a = np.eye(keep_dofs.size)
         D = self.stiffness - (2 * np.pi * frequency)**2 * self.mass
         D_dd = D[discard_dofs[:, np.newaxis],
@@ -1136,7 +1236,7 @@ class System:
             connection_indices = self.get_indices_by_coordinate(connection_degrees_of_freedom)
         else:
             connection_indices = np.array(connection_degrees_of_freedom)
-        other_indices = np.array([i for i in range(self.ndof) if not i in connection_indices])
+        other_indices = np.array([i for i in range(self.ndof) if i not in connection_indices])
 
         # Extract portions of the mass and stiffness matrices
         K_ii = self.K[other_indices[:, np.newaxis], other_indices]
@@ -1169,7 +1269,8 @@ class System:
     @classmethod
     def from_exodus_superelement(cls, superelement_nc4, transformation_exodus_file=None,
                                  x_disp='DispX', y_disp='DispY', z_disp='DispZ',
-                                 x_rot=None, y_rot=None, z_rot=None):
+                                 x_rot=None, y_rot=None, z_rot=None,
+                                 reduce_to_external_surfaces=False):
         """
         Creates a system from a superelement from Sierra/SD
 
@@ -1199,6 +1300,8 @@ class System:
         z_rot : str, optional
             Variable name to read for z-rotations in the transformation
             Exodus file. The default is to not read rotations.
+        reduce_to_external_surfaces : bool, optional
+            If True, exodus results will be reduced to external surfaces
 
         Raises
         ------
@@ -1248,6 +1351,8 @@ class System:
                 exo = transformation_exodus_file
             else:
                 raise ValueError('transformation_exodus_file must be a string or a sdpy.Exodus')
+            if reduce_to_external_surfaces:
+                exo = reduce_exodus_to_surfaces(exo, variables_to_transform=[var for var in [x_disp, y_disp, z_disp, x_rot, y_rot, z_rot] if var is not None])
             from .sdynpy_shape import ShapeArray
             shapes = ShapeArray.from_exodus(exo, x_disp, y_disp, z_disp, x_rot, y_rot, z_rot)
             transformation = shapes.shape_matrix.T
@@ -1256,6 +1361,47 @@ class System:
         system = cls(coordinates, Mr, Kr, Cr, transformation)
         return system, geometry, boundary_dofs
 
+    @classmethod
+    def from_sierra_sd_mfile_output(cls, maa_file, kaa_file, gid_file, aset_map_file):
+        """
+        Generates a System object from the MFILE, MAA, and KAA outputs from
+        Sierra/SD
+
+        Parameters
+        ----------
+        maa_file : str
+            Path to the file containing the MAA matrix.
+        kaa_file : str
+            Path to the file containing the KAA matrix.
+        gid_file : str
+            Path to the file contiaining the global node id map
+        aset_map_file : str
+            Path to the A-set Map.
+
+        Returns
+        -------
+        system : System
+            A system object with the specified matrices and degrees of freedom.
+
+        """
+        M = read_sierra_matlab_matrix_file(maa_file)
+        K = read_sierra_matlab_matrix_file(kaa_file)
+        gid = read_sierra_matlab_map_file(gid_file)
+        asetmap = read_sierra_matlab_map_file(aset_map_file).reshape(-1,9)
+        valid_dofs = asetmap != -1
+        # Check if any of the acoustic, temperature, or voltage are defined
+        if np.any(valid_dofs[:,-3:]):
+            warnings.warn('Sierra system contains acoustic, temperature, or voltage degrees of freedom.  These will be ignored by SDynPy.')
+        valid_dofs = valid_dofs[:,:-3]
+        asetmap = asetmap[:,:-3]
+        coordinates = coordinate_array(gid[:,np.newaxis],[1,2,3,4,5,6])[valid_dofs]
+        dof_ordering = asetmap[valid_dofs]-1
+        M = M[dof_ordering[:,np.newaxis],dof_ordering]
+        K = K[dof_ordering[:,np.newaxis],dof_ordering]
+        system = cls(coordinates,M,K)
+        return system
+
+
     def simulate_test(
             self,  # The system itself
             bandwidth,
@@ -1276,6 +1422,7 @@ class System:
             displacement_derivative=2,
             antialias_filter_cutoff_factor=3,
             antialias_filter_order=4,
+            multihammer_impact_spacing_factor = 4,
             **generator_kwargs
     ):
         available_excitations = ['pseudorandom', 'random',
@@ -1348,7 +1495,7 @@ class System:
             if num_signals > 1:
                 print(
                     'Warning: Hammer impact generally not recommended for multi-reference excitation, consider multi-hammer instead')
-            pulse_width = 2 / bandwidth
+            pulse_width = 2 / (bandwidth if excitation_max_frequency is None else excitation_max_frequency)
             signal_length = int(frame_length * integration_oversample * num_averages + (num_averages + 1)
                                 * extra_time_between_frames * sample_rate + 2 * pulse_width * sample_rate)
             pulse_times = np.arange(num_averages)[
@@ -1364,7 +1511,7 @@ class System:
             signals = np.tile(signals, [num_signals, 1])
         elif excitation.lower() == 'multi-hammer':
             signal_length = frame_length * integration_oversample
-            pulse_width = 2 / bandwidth
+            pulse_width = 2 / (bandwidth if excitation_max_frequency is None else excitation_max_frequency)
             signals = []
             for i in range(num_signals):
                 signals.append([])
@@ -1372,7 +1519,7 @@ class System:
                     pulse_times = []
                     last_pulse = 0
                     while last_pulse < frame_time * signal_fraction:
-                        next_pulse = last_pulse + pulse_width * (np.random.rand() * 4 + 1)
+                        next_pulse = last_pulse + pulse_width * (np.random.rand() * multihammer_impact_spacing_factor + 1)
                         pulse_times.append(next_pulse)
                         last_pulse = next_pulse
                     pulse_times = np.array(pulse_times)
@@ -1391,6 +1538,18 @@ class System:
                 signals[-1].append(np.zeros(int(extra_time_between_frames * sample_rate)))
                 signals[-1] = np.concatenate(signals[-1], axis=-1)
             signals = np.array(signals)
+        elif excitation.lower() == 'sine':
+            if num_signals > 1:
+                print(
+                    'Warning: Sine signal generally not recommended for multi-reference excitation')
+            frequencies = excitation_max_frequency if excitation_min_frequency is None else excitation_min_frequency
+            num_samples = frame_length * integration_oversample * num_averages + int(steady_state_time * sample_rate)
+            kwargs = {'frequencies': frequencies,
+                      'dt': dt,
+                      'num_samples': num_samples,
+                      'amplitudes': excitation_level}
+            kwargs.update(generator_kwargs)
+            signals = np.tile(generator.sine(**kwargs), (num_signals, 1))
         # Set up the integration
         responses, references = self.time_integrate(
             signals, dt, responses, references, displacement_derivative)
