@@ -170,7 +170,7 @@ class Shaker4DoF:
 
         Notes
         -----
-        The dof order of the responses are: 
+        The dof order of the responses are:
         Displacement of Armature,
         Displacement of Body,
         Displacement of Force Gauge,
@@ -220,10 +220,10 @@ class Shaker4DoF:
 
         Notes
         -----
-        The dof order of the state matrix is 
+        The dof order of the state matrix is
           0. armature displacement
           1. body displacement
-          2. forcegauge displacement, 
+          2. forcegauge displacement,
           3. armature velocity
           4. body velocity
           5. forcegauge velocity
@@ -257,8 +257,14 @@ class Shaker4DoF:
         A = np.array([[0, 0, 0, 1, 0, 0, 0],
                       [0, 0, 0, 0, 1, 0, 0],
                       [0, 0, 0, 0, 0, 1, 0],
-                      [(-self.k_suspension - self.k_stinger) / self.m_armature, self.k_suspension / self.m_armature, self.k_stinger / self.m_armature, (-self.c_suspension -
-                                                                                                                                                        self.c_stinger) / self.m_armature, self.c_suspension / self.m_armature, self.c_stinger / self.m_armature, self.force_factor / self.m_armature],
+                      [(-self.k_suspension - self.k_stinger) / self.m_armature,
+                       self.k_suspension / self.m_armature,
+                       self.k_stinger / self.m_armature,
+                       (-self.c_suspension
+                        - self.c_stinger) / self.m_armature,
+                       self.c_suspension / self.m_armature,
+                       self.c_stinger / self.m_armature,
+                       self.force_factor / self.m_armature],
                       [self.k_suspension / self.m_body, -self.k_suspension / self.m_body, 0, self.c_suspension /
                           self.m_body, -self.c_suspension / self.m_body, 0, -self.force_factor / self.m_body],
                       [self.k_stinger / self.m_forcegauge, 0, -self.k_stinger / self.m_forcegauge,
@@ -275,8 +281,10 @@ class Shaker4DoF:
             # States: x1, x2, x3, xd1, xd2, xd3, i
                      [1, 0, 0, 0, 0, 0, 0],  # Displacement of mass 1
                      [0, 0, 0, 1, 0, 0, 0],  # Velocity of mass 1
-                     [(-self.k_suspension - self.k_stinger) / self.m_armature, self.k_suspension / self.m_armature, self.k_stinger / self.m_armature, (-self.c_suspension - self.c_stinger) / \
-                      self.m_armature, self.c_suspension / self.m_armature, self.c_stinger / self.m_armature, self.force_factor / self.m_armature],  # Acceleration of mass 1
+                     [(-self.k_suspension - self.k_stinger) / self.m_armature, self.k_suspension / self.m_armature,
+                      self.k_stinger / self.m_armature, (-self.c_suspension - self.c_stinger) / \
+                      self.m_armature, self.c_suspension / self.m_armature,
+                      self.c_stinger / self.m_armature, self.force_factor / self.m_armature],  # Acceleration of mass 1
                      [0, 0, 0, 0, 0, 0, 0],  # External Force on mass 1
                      [0, 1, 0, 0, 0, 0, 0],  # Displacement of mass 2
                      [0, 0, 0, 0, 1, 0, 0],  # Velocity of mass 2
@@ -351,10 +359,10 @@ class Shaker4DoF:
         fig, ax = plt.subplots(2, 1, sharex=True)
         H = self.transfer_function(frequencies)[..., -1, -1]
         ax[0].plot(frequencies, np.real(1 / H))
-        if not test_data is None:
+        if test_data is not None:
             ax[0].plot(frequencies, np.real(test_data))
         ax[1].plot(frequencies, np.imag(1 / H))
-        if not test_data is None:
+        if test_data is not None:
             ax[1].plot(frequencies, np.real(test_data))
 
     @classmethod
