@@ -17,7 +17,11 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>."""
 from . import sdynpy_uff as uff
-unv = uff
-
 from . import sdynpy_rattlesnake as rattlesnake
 from . import sdynpy_vic as vic
+from . import sdynpy_tshaker as tshaker
+from . import sdynpy_pdf3D as pdf3D
+from . import sdynpy_escdf as escdf
+from .sdynpy_dataphysics import read_dataphysics_output
+
+unv = uff
