@@ -28,22 +28,27 @@ from ..fileio.sdynpy_rattlesnake import read_rattlesnake_output
 from ..core.sdynpy_geometry import Geometry
 from .sdynpy_smac import SMAC_GUI
 from .sdynpy_polypy import PolyPy_GUI
-from PyQt5 import QtWidgets, uic, QtGui
-from PyQt5.QtGui import QIcon, QFont
-from PyQt5.QtCore import Qt, QCoreApplication, QRect
-from PyQt5.QtWidgets import (QToolTip, QLabel, QPushButton, QApplication,
-                             QGroupBox, QWidget, QMessageBox, QHBoxLayout,
-                             QVBoxLayout, QSizePolicy, QMainWindow,
-                             QFileDialog, QErrorMessage, QListWidget, QListWidgetItem,
-                             QLineEdit,
-                             QDockWidget, QGridLayout, QButtonGroup, QDialog,
-                             QCheckBox, QRadioButton, QMenuBar, QMenu, QAction)
+from qtpy import QtWidgets, uic, QtGui
+from qtpy.QtGui import QIcon, QFont
+from qtpy.QtCore import Qt, QCoreApplication, QRect
+from qtpy.QtWidgets import (QToolTip, QLabel, QPushButton, QApplication,
+                            QGroupBox, QWidget, QMessageBox, QHBoxLayout,
+                            QVBoxLayout, QSizePolicy, QMainWindow,
+                            QFileDialog, QErrorMessage, QListWidget, QListWidgetItem,
+                            QLineEdit,
+                            QDockWidget, QGridLayout, QButtonGroup, QDialog,
+                            QCheckBox, QRadioButton, QMenuBar, QMenu)
+try:
+    from qtpy.QtGui import QAction
+except ImportError:
+    from qtpy.QtWidgets import QAction
 import numpy as np
 import pyqtgraph as pqtg
 import matplotlib.cm as cm
 from scipy.signal import get_window
 import traceback
 
+
 class SignalProcessingGUI(QMainWindow):
     """An iteractive window allowing users to compute FRFs"""
 
@@ -148,7 +153,7 @@ class SignalProcessingGUI(QMainWindow):
                        self.hysteresis_level_selector_references]:
             widget.setVisible(False)
         self.connect_callbacks()
-        if not time_history_array is None:
+        if time_history_array is not None:
             self.time_history_data = time_history_array.flatten()
             self.initialize_ui()
         self.setWindowTitle('Graphical Signal Processing Tool')
@@ -593,7 +598,7 @@ class SignalProcessingGUI(QMainWindow):
     def frequencyLinesChanged(self):
         print('Frequency Lines Changed')
         self.block_sampling_signals(True)
-        frame_size = (self.frequencyLinesSpinBox - 1) * 2
+        frame_size = (self.frequencyLinesSpinBox.value() - 1) * 2
         self.frameSizeSpinBox.setValue(frame_size)
         self.frameTimeDoubleSpinBox.setValue(frame_size / self.sample_rate)
         self.frequencySpacingDoubleSpinBox.setValue(self.sample_rate / frame_size)
@@ -677,7 +682,7 @@ class SignalProcessingGUI(QMainWindow):
             pen = pqtg.mkPen(color=[int(255 * v) for v in self.cm(j % self.cm_mod)])
             self.referencesPlot.plot(x=data_entry.abscissa,
                                      y=data_entry.ordinate, name=text, pen=pen)
-        if not xrange is None:
+        if xrange is not None:
             self.referencesPlot.setXRange(*xrange, padding=0.0)
         self.referencesPlot.addItem(self.time_selector_references)
         self.referencesPlot.addItem(self.trigger_level_selector_references)
@@ -711,7 +716,7 @@ class SignalProcessingGUI(QMainWindow):
             # print(pen)
             self.responsesPlot.plot(x=data_entry.abscissa,
                                     y=data_entry.ordinate, name=text, pen=pen)
-        if not xrange is None:
+        if xrange is not None:
             self.responsesPlot.setXRange(*xrange, padding=0.0)
         self.responsesPlot.addItem(self.time_selector_responses)
         self.responsesPlot.addItem(self.trigger_level_selector_responses)
@@ -922,7 +927,7 @@ class SignalProcessingGUI(QMainWindow):
             frequency_spacing = self.frequencySpacingDoubleSpinBox.value()
             frame_size = self.frameSizeSpinBox.value()
             frame_indices = np.array([start_index for frame_number, start_index in enumerate(self.frame_start_indices)
-                                      if not frame_number in self.ignore_frames])[:, np.newaxis] + np.arange(frame_size)
+                                      if frame_number not in self.ignore_frames])[:, np.newaxis] + np.arange(frame_size)
             if frame_indices.shape[0] == 0:
                 QMessageBox.critical(self, 'Invalid Frames',
                                      'At least one measurement frame must be selected')
@@ -1005,7 +1010,7 @@ class SignalProcessingGUI(QMainWindow):
                     self.frf_data = data_array(FunctionTypes.FREQUENCY_RESPONSE_FUNCTION,
                                                freq, np.moveaxis(H, 0, -1), frf_coordinate)
                     success = True
-                elif self.frfComboBox.currentIndex() == 1: # H2
+                elif self.frfComboBox.currentIndex() == 1:  # H2
                     if (response_fft.shape != reference_fft.shape):
                         QMessageBox.critical(self, 'Bad FRF Shape',
                                              'For H2, Number of inputs must equal number of outputs')
@@ -1020,7 +1025,7 @@ class SignalProcessingGUI(QMainWindow):
                         self.frf_data = data_array(FunctionTypes.FREQUENCY_RESPONSE_FUNCTION,
                                                    freq, np.moveaxis(H, 0, -1), frf_coordinate)
                         success = True
-                elif self.frfComboBox.currentIndex() == 2: # H3
+                elif self.frfComboBox.currentIndex() == 2:  # H3
                     if (response_fft.shape != reference_fft.shape):
                         QMessageBox.critical(self, 'Bad FRF Shape',
                                              'For H3, Number of inputs must equal number of outputs')
@@ -1040,7 +1045,7 @@ class SignalProcessingGUI(QMainWindow):
                         self.frf_data = data_array(FunctionTypes.FREQUENCY_RESPONSE_FUNCTION,
                                                    freq, np.moveaxis(H, 0, -1), frf_coordinate)
                         success = True
-                elif self.frfComboBox.currentIndex() == 3: # Hv
+                elif self.frfComboBox.currentIndex() == 3:  # Hv
                     Gxx = np.einsum('...iaf,...iaf->...if', response_fft,
                                     np.conj(response_fft))[..., np.newaxis, np.newaxis]
                     Gxf = np.einsum('...iaf,...jaf->...ifj', response_fft,
@@ -1051,7 +1056,7 @@ class SignalProcessingGUI(QMainWindow):
                     # print(Gxf.shape)
                     # print(Gff.shape)
                     # Broadcast over all responses
-                    Gff = np.broadcast_to(Gff,Gxx.shape[:-2]+Gff.shape[-2:])
+                    Gff = np.broadcast_to(Gff, Gxx.shape[:-2]+Gff.shape[-2:])
                     Gffx = np.block([[Gff, np.conj(np.moveaxis(Gxf, -2, -1))],
                                      [Gxf, Gxx]])
                     # Compute eigenvalues
@@ -1145,7 +1150,7 @@ class SignalProcessingGUI(QMainWindow):
 
     def saveWindowedTimeHistory(self):
         filename, file_filter = QtWidgets.QFileDialog.getSaveFileName(
-            self, 'Select File to Save Windowed Time Data', filter='Numpy File (*.npy)')
+            self, 'Select File to Save Windowed Time Data', filter='Numpy File (*.npz)')
         if filename == '':
             return
         self.windowed_time_data.save(filename)
@@ -1155,7 +1160,7 @@ class SignalProcessingGUI(QMainWindow):
 
     def saveFRF(self):
         filename, file_filter = QtWidgets.QFileDialog.getSaveFileName(
-            self, 'Select File to Save FRF Data', filter='Numpy File (*.npy)')
+            self, 'Select File to Save FRF Data', filter='Numpy File (*.npz)')
         if filename == '':
             return
         self.frf_data.save(filename)
@@ -1165,7 +1170,7 @@ class SignalProcessingGUI(QMainWindow):
 
     def saveCoherence(self):
         filename, file_filter = QtWidgets.QFileDialog.getSaveFileName(
-            self, 'Select File to Save Coherence Data', filter='Numpy File (*.npy)')
+            self, 'Select File to Save Coherence Data', filter='Numpy File (*.npz)')
         if filename == '':
             return
         self.coherence_data.save(filename)
@@ -1175,7 +1180,7 @@ class SignalProcessingGUI(QMainWindow):
 
     def saveAutospectra(self):
         filename, file_filter = QtWidgets.QFileDialog.getSaveFileName(
-            self, 'Select File to Save Autospectra Data', filter='Numpy File (*.npy)')
+            self, 'Select File to Save Autospectra Data', filter='Numpy File (*.npz)')
         if filename == '':
             return
         self.autospectra_data.save(filename)
@@ -1185,7 +1190,7 @@ class SignalProcessingGUI(QMainWindow):
 
     def saveCrossspectra(self):
         filename, file_filter = QtWidgets.QFileDialog.getSaveFileName(
-            self, 'Select File to Save Crosspectra Data', filter='Numpy File (*.npy)')
+            self, 'Select File to Save Crosspectra Data', filter='Numpy File (*.npz)')
         if filename == '':
             return
         self.crossspectra_data.save(filename)
@@ -1239,10 +1244,10 @@ class SignalProcessingGUI(QMainWindow):
     def loadData(self):
         try:
             filename, file_filter = QFileDialog.getOpenFileName(
-                self, 'Select Time History File', filter='Numpy (*.npy);;Rattlesnake (*.nc4)')
+                self, 'Select Time History File', filter='Numpy (*.npz);;Rattlesnake (*.nc4)')
             if filename == '':
                 return
-            if file_filter == 'Numpy (*.npy)':
+            if file_filter == 'Numpy (*.npz)':
                 self.time_history_data = TimeHistoryArray.load(filename)
                 self.initialize_ui()
             elif file_filter == 'Rattlesnake (*.nc4)':
