@@ -18,6 +18,7 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>."""
 from . import sdynpy_cpsd as cpsd
 from . import sdynpy_frf as frf
+from . import sdynpy_lrm as lrm
 from . import sdynpy_generator as generator
 from . import sdynpy_integration as integration
 from . import sdynpy_correlation as correlation
@@ -25,3 +26,7 @@ from . import sdynpy_complex as complex
 from . import sdynpy_rotation as rotation
 from . import sdynpy_camera as camera
 from . import sdynpy_harmonic as harmonic
+from . import sdynpy_geometry_fitting as geometry_fitting
+from . import sdynpy_frf_inverse as frf_inverse
+from . import sdynpy_srs as srs
+from . import sdynpy_buffer as buffer
