@@ -23,6 +23,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 import numpy as np
 import scipy.signal as sig
 import matplotlib.pyplot as plt
+from .sdynpy_lrm import frf_local_model
 
 
 def sysmat2frf(frequencies, M, C, K, frf_type='disp'):
@@ -93,7 +94,7 @@ def sysmat2frf(frequencies, M, C, K, frf_type='disp'):
     return H
 
 
-def modes2frf(frequencies, natural_frequencies, damping_ratios, mode_shapes,
+def modes2frf(frequencies, natural_frequencies, damping_ratios, mode_shapes=None,
               input_mode_shapes=None, frf_type='disp'):
     '''Compute Frequency Response Functions given modal properties
 
@@ -112,17 +113,19 @@ def modes2frf(frequencies, natural_frequencies, damping_ratios, mode_shapes,
     damping_ratios : ndarray
         Critical damping ratios of the structure in ratio form rather than
         percentange, e.g. 2% damping would be specified as 0.02 rather than 2.
-    mode_shapes : ndarray
+    mode_shapes : ndarray, optional
         A 2D mode shape matrix with shape (no,nm) where no is the number of
         output responses and nm is the number of modes.  If the optional
         argument input_mode_shapes is not specified, this mode shape matrix
         will also be used for the inputs, resulting in a square FRF matrix.
-    input_mode_shapes : ndarray
+        If mode_shapes is not specified, the values of the modal FRF matrix
+        are returned as 2D array.
+    input_mode_shapes : ndarray, optional
         A 2D mode shape matrix with shape (ni,nm) where ni is the number of
         input forces and nm is the number of modes.  If the optional argument 
         input_mode_shapes is specified, it be used for the inputs, resulting in
         a potentially nonsquare FRF matrix.
-    frf_type : str
+    frf_type : str, optional
         frf_type should be one of ['disp','vel','accel'] or ['displacement',
         'velocity','acceleration'] to specify which "type" of frequency
         response function to compute.  By default it computes a displacement or
@@ -134,9 +137,10 @@ def modes2frf(frequencies, natural_frequencies, damping_ratios, mode_shapes,
     Returns
     -------
     H : ndarray
-        A 3D np array with shape (nf,no,ni), where nf is the number of
-        frequency lines, no is the number of outputs, and ni is the number of
-        inputs.  Values in H are complex.
+        A 3D (or 2D) np array depending on whether or not mode_shapes is 
+        defined with shape (nf,no,ni) or (nf,nm), where nf is the number of
+        frequency lines, no is the number of outputs, ni is the number of
+        inputs, and nm is the number of modes.  Values in H are complex.
 
     Notes
     -----
@@ -152,25 +156,32 @@ def modes2frf(frequencies, natural_frequencies, damping_ratios, mode_shapes,
          + 1j * (2 * np.pi * frequencies[:, np.newaxis]) * 2 *
          damping_ratios * (2 * np.pi * natural_frequencies)
          + (2 * np.pi * natural_frequencies)**2)
-
-    if input_mode_shapes is None:
-        input_mode_shapes = mode_shapes
-
-    H = np.einsum('ij,fj,lj->fil', mode_shapes, 1 / Z, input_mode_shapes)
+    if mode_shapes is not None:
+        if input_mode_shapes is None:
+            input_mode_shapes = mode_shapes
+        H = np.einsum('ij,fj,lj->fil', mode_shapes, 1 / Z, input_mode_shapes)
+    else:
+        H = 1/Z
 
     if frf_type in ['vel', 'velocity']:
-        H = 1j * (2 * np.pi * frequencies[:, np.newaxis, np.newaxis]) * H
+        if np.ndim(H) == 3:
+            H = 1j * (2 * np.pi * frequencies[:, np.newaxis, np.newaxis]) * H
+        elif np.ndim(H) == 2:
+            H = 1j * (2 * np.pi * frequencies[:, np.newaxis]) * H
+        # num = [1,0]
     elif frf_type in ['accel', 'acceleration']:
-        H = -(2 * np.pi * frequencies[:, np.newaxis, np.newaxis])**2 * H
-
+        if np.ndim(H) == 3:
+            H = -(2 * np.pi * frequencies[:, np.newaxis, np.newaxis])**2 * H
+        elif np.ndim(H) == 2:
+            H = -(2 * np.pi * frequencies[:, np.newaxis])**2 * H
     return H
 
-
 def timedata2frf(references, responses, dt=1, samples_per_average=None,
                  overlap=0.0, method='H1', window=np.array((1.0,)),
                  response_fft=lambda array: np.fft.rfft(array, axis=-1),
                  reference_fft=lambda array: np.fft.rfft(array, axis=-1),
-                 response_fft_array=None, reference_fft_array=None):
+                 response_fft_array=None, reference_fft_array=None,
+                 return_model_data=False, **lrm_kwargs):
     # TODO Update DOCSTRING with changes after they are all made.
     '''Creates an FRF matrix given time histories of responses and references
 
@@ -196,7 +207,7 @@ def timedata2frf(references, responses, dt=1, samples_per_average=None,
     overlap : float
         The overlap as a fraction of the frame (e.g. 0.5 specifies 50% overlap).
         If not specified, no overlap is used.
-    method : str in ['H1','H2','Hv','Hs']
+    method : str in ['H1','H2','Hv','Hs','LRM']
         The method for creating the frequency response function. 'H1' is 
         default if not specified.
     window : ndarray or str
@@ -211,6 +222,15 @@ def timedata2frf(references, responses, dt=1, samples_per_average=None,
     reference_fft_array : np.ndarray
         Array to store the data into before taking the FFT.  Should be
         size number_of_references, n_averages, samples_per_average
+    return_model_data : boolean
+        Wheter to return selected model orders used in the 'LRM' method.
+        default is False        
+    **lrm_kwargs
+        Additional keyword arguments specify parameters if method == 'LRM'.
+        Possible arguments are: f_out (ndarray), bandwidth (float), transient
+        (boolean), modelset (iterable of (3,1) ndarray), export_ratio (float), 
+        max_parallel (int), print_time (bool). See sdynpy_lrm.frf_local_model.
+        samples_per_average, overlap, and window are void if method=='LRM'.
 
     Returns
     -------
@@ -219,7 +239,9 @@ def timedata2frf(references, responses, dt=1, samples_per_average=None,
     H : ndarray
         A nf x no x ni array where nf is the number of frequency lines, no is
         the number of outputs, and ni is the number of inputs.
-
+    model_data: dict
+        Contains None if method != 'LRM'. See sdynpy_lrm.frf_local_model.
+        
     Notes
     -----
     There are requirements for the shapes of references and responses for some
@@ -229,7 +251,7 @@ def timedata2frf(references, responses, dt=1, samples_per_average=None,
     if references.shape[-1] != responses.shape[-1]:
         raise ValueError(
             'reference and responses must have the same number of time steps (last dimension)!')
-    if not method in ['H1', 'H2', 'H3', 'Hs', 'Hv', 'Hcd']:
+    if not method in ['H1', 'H2', 'H3', 'Hs', 'Hv', 'Hcd','LRM']:
         raise ValueError('method parameter must be one of ["H1","H2","H3","Hv","Hcd"]')
     if references.ndim == 1:
         references = references[np.newaxis, :]
@@ -237,12 +259,14 @@ def timedata2frf(references, responses, dt=1, samples_per_average=None,
         responses = responses[np.newaxis, :]
     # Set up time indices
     ntimes_total = references.shape[-1]
-    if samples_per_average is None:
+    if samples_per_average is None or method == 'LRM':
         samples_per_average = ntimes_total
     overlap_samples = int(samples_per_average * overlap)
     time_starts = np.arange(0, ntimes_total - samples_per_average +
                             1, samples_per_average - overlap_samples)
     time_indices_for_averages = time_starts[:, np.newaxis] + np.arange(samples_per_average)
+    if method == 'LRM':
+        window=np.array((1.0,)) # local modeling does not use windowing
     if isinstance(window, str):
         window = sig.windows.get_window(window.lower(), samples_per_average, fftbins=True)
     # Sort the references and responses into their time arrays
@@ -258,7 +282,7 @@ def timedata2frf(references, responses, dt=1, samples_per_average=None,
     frequencies = np.fft.rfftfreq(samples_per_average, dt)
     response_ffts = response_fft(response_fft_array)
     reference_ffts = reference_fft(reference_fft_array)
-    # Now start computing FFTs
+    # Now start computing FRFs
     if method == 'H1':
         # We want to compute X*F^H = [X1;X2;X3][F1^H F2^H F3^H]
         Gxf = np.einsum('...iaf,...jaf->...fij', response_ffts, np.conj(reference_ffts))
@@ -317,12 +341,21 @@ def timedata2frf(references, responses, dt=1, samples_per_average=None,
         evect = evect[..., 0]  # Assumes evals are sorted ascending
         H = np.moveaxis(-evect[..., :-1] / evect[..., -1:],  # Scale so last value is -1
                         -3, -2)
+    elif method == 'LRM':
+        frequencies, H, model_data = frf_local_model(reference_ffts,response_ffts,
+                                                         frequencies,**lrm_kwargs)
     else:
         raise NotImplementedError('Method {:} has not been implemented yet!'.format(method))
-    return frequencies, H
+        
+    if return_model_data:
+        if method != 'LRM':
+            model_data = {'info': None,'modelset': None,'model_selected': None}
+        return frequencies, H, model_data
+    else:
+        return frequencies, H
 
 
-def fft2frf(references, responses, method='H1'):
+def fft2frf(references, responses, method='H1',freqs_in=None,**kwargs):
     '''Creates an FRF matrix given ffts of responses and references
 
     This function creates a nf x no x ni FRF matrix from the ffts
@@ -336,8 +369,15 @@ def fft2frf(references, responses, method='H1'):
     responses : ndarray
         A no x nf or nf array where no is the number of responses and nf is the
         number of time frequencies in the fft.
-    method : str in ['H1','H2','Hv','Hs']
+    method : str in ['H1','H2','Hv','Hs','LRM']
         The method for creating the frequency response function.
+    freqs_in : ndarray
+        Only used if method == 'LRM'. A nf or nf x 1 array of frquency values.
+    **lrm_kwargs
+        Additional keyword arguments specify parameters if method == 'LRM'.
+        Possible arguments are: f_out (ndarray), bandwidth (float), transient
+        (boolean), modelset (iterable of (3,1) ndarray), max_parallel (int), 
+        export_ratio (float). See sdynpy_lrm.frf_local_model.
 
     Returns
     -------
@@ -345,6 +385,10 @@ def fft2frf(references, responses, method='H1'):
         A nf x no x ni array where nf is the number of frequency lines, no is
         the number of outputs, and ni is the number of inputs.  The output
         frequency lines will correspond to the frequency lines in the ffts.
+    freqs_out : None or ndarray
+        None unless method == 'LRM'. See sdynpy_lrm.frf_local_model.
+    model_data: None dict
+        None unless method == 'LRM'. See sdynpy_lrm.frf_local_model.
 
     Notes
     -----
@@ -360,8 +404,8 @@ def fft2frf(references, responses, method='H1'):
         responses = responses[np.newaxis, :]
     elif responses.ndim > 2:
         raise ValueError('responses should be at maximum a 2 dimensional array')
-    if not method in ['H1', 'H2', 'Hs', 'Hv']:
-        raise ValueError('method parameter must be one of ["H1","H2","Hs","Hv"]')
+    if not method in ['H1', 'H2', 'Hs', 'Hv', 'LRM']:
+        raise ValueError('method parameter must be one of ["H1","H2","Hs","Hv","LRM"]')
 #    H = np.zeros((references.shape[1],responses.shape[0],references.shape[0]),dtype=complex)
 #    for i in range(H.shape[1]):
 #            for j in range(H.shape[2]):
@@ -373,9 +417,18 @@ def fft2frf(references, responses, method='H1'):
         Gxf = np.einsum('if,jf->fij', responses, references.conj())
         Gff = np.einsum('if,jf->fij', references, references.conj())
         H = np.linalg.solve(Gff.transpose(0, 2, 1), Gxf.transpose((0, 2, 1))).transpose((0, 2, 1))
+    elif method == 'LRM':
+        if freqs_in is None:
+            raise ValueError('if method == \'LRM\', freqs_in must be specified')
+        import sdynpy.signal_processing.sdynpy_lrm as frflm
+        freqs_out, H, model_data = frflm.frf_local_model(references, responses, 
+                                                         freqs_in, **lrm_kwargs)
     else:
         raise NotImplementedError('Method {:} has not been implemented'.format(method))
-    return H
+    if method != 'LRM':
+        freqs_out = None
+        model_data = None
+    return H, freqs_out, model_data
 
 
 def plot(H, f, responses=None, references=None, real_imag=False):
